@@ -8,6 +8,8 @@
 #include <sys/types.h>
 #include <regex.h>
 
+#include "stack_ptr.hpp"
+#include "cache-t.hpp"
 #include "string.hpp"
 #include "vector.hpp"
 #include "config.hpp"
@@ -19,14 +21,9 @@
 #include "file_util.hpp"
 #include "fstream.hpp"
 #include "getdata.hpp"
-#include "directory.hpp"
 #include "strtonum.hpp"
 #include "asc_ctype.hpp"
 #include "iostream.hpp"
-#include <stdio.h>
-
-#define DEBUG {fprintf(stderr,"File: %s(%i)\n",__FILE__,__LINE__);}
-
 
 namespace acommon {
 
@@ -66,7 +63,7 @@ namespace acommon {
     void setDescription(const String & desc) {desc_ = desc;}
     const String & getDescription() {return desc_;}
     PosibErr<void> expand(Config * config);
-    PosibErr<void> build(FILE * in, Config * config, int line = 1, 
+    PosibErr<void> build(FStream &, int line = 1, 
                          const char * name = "mode file");
 
     ~FilterMode();
@@ -85,6 +82,47 @@ namespace acommon {
     Vector<KeyValue> expansion;
   };
 
+  class FilterModeList : public Cacheable, public Vector<FilterMode>
+  {
+  public:
+    typedef Config CacheConfig;
+    typedef String CacheKey;
+    String key;
+    static PosibErr<FilterModeList *> get_new(const String & key, const Config *);
+    bool cache_key_eq(const String & okey) const {
+      return key == okey;
+    }
+  };
+
+  class ModeNotifierImpl : public Notifier
+  {
+  private:
+    ModeNotifierImpl();
+    ModeNotifierImpl(const ModeNotifierImpl &);
+    ModeNotifierImpl & operator= (const ModeNotifierImpl & b);
+    CachePtr<FilterModeList> filter_modes_;
+  public:
+    Config * config;
+    PosibErr<FilterModeList *> get_filter_modes();
+    
+    ModeNotifierImpl(Config * c) : config(c) 
+    {
+      c->filter_mode_notifier = this;
+    }
+    ModeNotifierImpl(const ModeNotifierImpl & other,  Config * c) 
+      : filter_modes_(other.filter_modes_), config(c) 
+    {
+      c->filter_mode_notifier = this;
+    }
+    
+    ModeNotifierImpl * clone(Config * c) const {return new ModeNotifierImpl(*this, c);}
+
+    PosibErr<void> item_updated(const KeyInfo * ki, ParmStr);
+    PosibErr<void> list_updated(const KeyInfo * ki);
+
+    ~ModeNotifierImpl() {}
+  };
+
   FilterMode::FilterMode(const String & name)
   : name_(name) {}
 
@@ -166,7 +204,7 @@ namespace acommon {
   bool FilterMode::lockFileToMode(const String & fileName,FILE * in) {
 
     Vector<unsigned int> extStart;
-    int first_point = fileName.length();
+    int first_point = fileName.size();
 
     while ( first_point > 0 ) {
       while (    ( --first_point >= 0 )
@@ -268,32 +306,32 @@ namespace acommon {
 
   PosibErr<bool> FilterMode::MagicString::testMagic(FILE * seekIn,String & magic,const String & mode) {
 
-    if ( magic.length() == 0 ) {
+    if ( magic.size() == 0 ) {
       return true;
     }
  
     unsigned int magicFilePosition = 0;
 
-    while (    ( magicFilePosition < magic.length() )
+    while (    ( magicFilePosition < magic.size() )
             && ( magic[magicFilePosition] != ':' ) ) {
       magicFilePosition++;
     }
 
     String number(magic);
 
-    number.erase(magicFilePosition,magic.length() - magicFilePosition);
+    number.erase(magicFilePosition,magic.size() - magicFilePosition);
 
     char * num = (char *)number.str();
-    char * numEnd = num + number.length();
+    char * numEnd = num + number.size();
     char * endHere = numEnd;
     long position = 0;
 
-    if (    ( number.length() == 0 ) 
+    if (    ( number.size() == 0 ) 
          || ( (position = strtoi_c(num,&numEnd)) < 0 )
          || ( numEnd != endHere ) ) {
       return make_err(file_magic_pos,"",magic.str());
     }
-    if (    ( magicFilePosition >= magic.length() )
+    if (    ( magicFilePosition >= magic.size() )
          || (    ( seekIn != NULL )
               && ( fseek(seekIn,position,SEEK_SET) < 0 ) ) ) {
       if ( seekIn != NULL ) {
@@ -305,7 +343,7 @@ namespace acommon {
     //increment magicFilePosition to skip the `:'
     unsigned int seekRangePos = ++ magicFilePosition; 
 
-    while (    ( magicFilePosition < magic.length() )
+    while (    ( magicFilePosition < magic.size() )
             && ( magic[magicFilePosition] != ':' ) ) {
       magicFilePosition++;
     }
@@ -313,7 +351,7 @@ namespace acommon {
     String magicRegExp(magic);
 
     magicRegExp.erase(0,magicFilePosition + 1);
-    if ( magicRegExp.length() == 0 ) {
+    if ( magicRegExp.size() == 0 ) {
       if ( seekIn != NULL ) {
         rewind(seekIn);
       }
@@ -321,12 +359,12 @@ namespace acommon {
     }
     
     number = magic;
-    number.erase(magicFilePosition,magic.length() - magicFilePosition);
+    number.erase(magicFilePosition,magic.size() - magicFilePosition);
     number.erase(0,seekRangePos);//already incremented by one see above
     num = (char*)number.str();
-    endHere = numEnd = num + number.length();
+    endHere = numEnd = num + number.size();
 
-    if (    ( number.length() == 0 )
+    if (    ( number.size() == 0 )
          || ( (position = strtoi_c(num,&numEnd)) < 0 )
          || ( numEnd != endHere ) ) {
       if ( seekIn != NULL ) {
@@ -345,7 +383,6 @@ namespace acommon {
       }
 
       char regError[256];
-      CERR.printl(magicRegExp.str());
       regerror(regsucess,&seekMagic,&regError[0],256);
       return make_err(bad_magic,mode.str(),magic.str(),regError);
     }
@@ -382,25 +419,21 @@ namespace acommon {
 
   PosibErr<void> FilterMode::expand(Config * config) {
 
-    config->replace("rem-all-filter","");
+    config->replace("clear-filter","");
     for ( Vector<KeyValue>::iterator it = expansion.begin() ;
-          it != expansion.end() ; it++ ) {
-
-      PosibErr<void> repErr = config->replace(it->key, it->value);
-
-      if ( repErr.has_err() ) {
-        return repErr.with_file(file_);
-      }      
+          it != expansion.end() ; it++ ) 
+    {
+      PosibErr<void> pe = config->replace(it->key, it->value);
+      if (pe.has_err()) return pe.with_file(file_);
     }
     return no_err;  
   }
 
-  PosibErr<void> FilterMode::build(FILE * in,Config * config, int line0, const char * name) {
+  PosibErr<void> FilterMode::build(FStream & toParse, int line0, const char * name) {
 
     String buf;
     DataPair dp;
     dp.line_num = line0;
-    FStream toParse(in,false);
 
     while ( getdata_pair(toParse, dp, buf) ) {
 
@@ -426,276 +459,135 @@ namespace acommon {
 
     return no_err;
   }
-    
-  static Vector<FilterMode> filterModes;
-  int filterModesRef = 0;
 
+  static GlobalCache<FilterModeList> filter_modes_cache("filter_modes");
 
+  PosibErr<void> set_mode_from_extension (Config * config, ParmString filename, FILE * in) 
+  {
+    RET_ON_ERR_SET(static_cast<ModeNotifierImpl *>(config->filter_mode_notifier)
+                   ->get_filter_modes(), FilterModeList *, fm);
 
-  void set_mode_from_extension (Config * config, ParmString filename, FILE * in) {
-    for ( Vector<FilterMode>::iterator it = filterModes.begin() ;
-         it != filterModes.end() ; it++ ) {
+    for ( FilterModeList::iterator it = fm->begin(); it != fm->end(); it++ ) 
+    {
       if ( it->lockFileToMode(filename,in) ) {
         config->replace("mode", it->modeName().str());
         break;
       }
     }
+    return no_err;
   }
 
-
   void activate_filter_modes(Config *config);
 
-
-  class ModeNotifierImpl : public Notifier
+  PosibErr<FilterModeList *>  ModeNotifierImpl::get_filter_modes()
   {
-  private:
-    Config * config;
-    StringList mode_path;
-
-    ModeNotifierImpl();
-    ModeNotifierImpl(const ModeNotifierImpl &);
-    ModeNotifierImpl & operator= (const ModeNotifierImpl & b);
-    PosibErr<void> intLoadModes(const bool reset = false);
-  public:
-    ModeNotifierImpl(Config * c);
-    
-    ModeNotifierImpl * clone(Config * c) const {return new ModeNotifierImpl(c);}
-
-    PosibErr<void> item_updated(const KeyInfo * ki, ParmString value);
-    PosibErr<void> item_added(const KeyInfo * ki, ParmString value);
-
-    PosibErr<void> initModes();
-
-    ~ModeNotifierImpl();
-  };
-
-  PosibErr<void> ModeNotifierImpl::initModes() {
-
-    if (    ( filterModesRef > 0 )
-         && ( filterModes.size() > 0 ) ) {
-      return no_err;
+    if (!filter_modes_) {
+      //FIXME is filter-path proper for filter mode files ???
+      //      if filter-options-path better ???
+      //      do we need a filter-mode-path ???
+      //      should change to use genetic data-path once implemented
+      //        and then search filter-path - KevinA
+      String filter_path;
+      StringList filter_path_lst;
+      config->retrieve_list("filter-path", &filter_path_lst);
+      combine_list(filter_path, filter_path_lst);
+      RET_ON_ERR(setup(filter_modes_, &filter_modes_cache, config, filter_path));
     }
-    filterModes.clear();
-    return intLoadModes();
+    return filter_modes_.get();
   }
 
-  ModeNotifierImpl::ModeNotifierImpl() 
-  : config(NULL),
-    mode_path()
+
+  PosibErr<void> ModeNotifierImpl::item_updated(const KeyInfo * ki, ParmStr value)
   {
-    filterModesRef++;
-  }
-
-  ModeNotifierImpl::ModeNotifierImpl(Config * c) 
-  : config(c),
-    mode_path()
-  {
-    filterModesRef++;
-  }
-
-  ModeNotifierImpl::ModeNotifierImpl(const ModeNotifierImpl & b) 
-  : config(b.config),
-    mode_path(b.mode_path)
-  {
-    filterModesRef++;
-  }
-
-  ModeNotifierImpl::~ModeNotifierImpl() {
-    if ( --filterModesRef < 1 ) {
-      filterModes.clear();
-    }
-  }
-    
-  ModeNotifierImpl & ModeNotifierImpl::operator= (const ModeNotifierImpl & b) {
-    config = b.config;
-    mode_path = b.mode_path;
-    return *this;
-  }
-
-  PosibErr<void> ModeNotifierImpl::item_updated(const KeyInfo * ki, ParmString value) {
-    if ( strcmp(ki->name, "-t" ) == 0 ) {
-      return (config->replace("mode","tex"));
-    }
-    if ( strcmp(ki->name, "-H" ) == 0 ) {
-      return (config->replace("mode","html"));
-    }
-    if ( strcmp(ki->name, "-e" ) == 0 ) {
-
-      return (config->replace("mode","email"));
-    }
     if ( strcmp(ki->name, "mode") == 0 ) {
-      for ( Vector<FilterMode>::iterator it = filterModes.begin() ;
-            it != filterModes.end() ; it++ ) {
-        if ( it->modeName() == value ) {
-          config->replace("rem-all-filter","");
+      RET_ON_ERR_SET(get_filter_modes(), FilterModeList *, filter_modes);
+      for ( Vector<FilterMode>::iterator it = filter_modes->begin() ;
+            it != filter_modes->end() ; it++ ) {
+        if ( it->modeName() == value )
           return it->expand(config);
-        }
       }
-      return make_err(unknown_mode,value); 
+      return make_err(unknown_mode, value); 
     }
     return no_err;
   }
 
-  PosibErr<void> ModeNotifierImpl::item_added(const KeyInfo * ki, ParmString value) {
-  
-    if ( strcmp(ki->name, "filter-path") == 0) {
-      return intLoadModes();
+  PosibErr<void> ModeNotifierImpl::list_updated(const KeyInfo * ki)
+  {
+    if (strcmp(ki->name, "filter-path") == 0) {
+      filter_modes_.reset(0);
     }
     return no_err;
   }
 
-  PosibErr<void> ModeNotifierImpl::intLoadModes(const bool reset) {
+  PosibErr<FilterModeList *> FilterModeList::get_new(const String & key,
+                                                     const Config *) 
+  {
 
-
-//FIXME is filter-path proper for filter mode files ???
-//      if filter-options-path better ???
-//      do we need a filter-mode-path ???
-    RET_ON_ERR(config->retrieve_list("filter-path",&mode_path));
-    if ( mode_path.elements()->at_end() ) {
-      return no_err;
-    }
+    StackPtr<FilterModeList> filter_modes(new FilterModeList);
+    filter_modes->key = key;
+    StringList mode_path;
+    separate_list(key, mode_path);
     
-    PathBrowser mode_files(mode_path);
-    regex_t seekfor;
-
-    int reerr = 0;
-
-//FIXME reset regexp to default possix 
-    reerr = regcomp(&seekfor,"\\w+\\.amf$",
-                    REG_NEWLINE|REG_NOSUB|REG_ICASE|REG_EXTENDED);
-    assert(reerr == 0);
+    PathBrowser els(mode_path, ".amf");
 
     String possMode;
+    String possModeFile;
 
-    while ( mode_files.expand_file_part(&seekfor,possMode) ) {
+    const char * file;
+    while ((file = els.next()) != NULL) 
+    {
+      possModeFile = file;
+      possMode.assign(possModeFile.str(), possModeFile.size() - 4);
 
-      String possModeFile = possMode;
+      unsigned pathPos = 0;
+      unsigned pathPosEnd = 0;
 
-      possMode.erase(possMode.length() - 4,4);
-      
-      int pathPos = 0;
-      int pathPosEnd = 0;
-
-      while (    ( (pathPosEnd = possMode.find('/',pathPos)) < possMode.length() )
+      while (    ( (pathPosEnd = possMode.find('/',pathPos)) < possMode.size() )
               && ( pathPosEnd >= 0 ) ) {
         pathPos = pathPosEnd + 1;
       }
       possMode.erase(0,pathPos);
+      possMode.ensure_null_end();
+      to_lower(possMode.data());
 
-      Vector<FilterMode>::iterator fmIt = filterModes.begin();
+      Vector<FilterMode>::iterator fmIt = filter_modes->begin();
 
-      for ( fmIt = filterModes.begin() ; 
-            fmIt != filterModes.end() ; fmIt++ ) {
+      for ( fmIt = filter_modes->begin() ; 
+            fmIt != filter_modes->end() ; fmIt++ ) {
         if ( (*fmIt).modeName() == possMode ) {
           break;
         }
       }
-      if ( fmIt != filterModes.end() ) {
+      if ( fmIt != filter_modes->end() ) {
         continue;
       }
 
-      FILE * in = NULL;
+      FStream toParse;
 
-      if ( (in = fopen(possModeFile.str(),"rb")) == NULL ) {
-        //FIXME is it desired to issue an warning if file can not be read ?
-        //      don't think so.
-        continue;
-      }
+      RET_ON_ERR(toParse.open(possModeFile.str(),"rb"));
 
       String buf;
       DataPair dp;
 
-      FStream toParse(in,false);
-
       bool get_sucess = getdata_pair(toParse, dp, buf);
       
       to_lower(dp.key);
       to_lower(dp.value);
       if (    !get_sucess
            || ( dp.key != "mode" ) 
-           || ( dp.value != possMode.lower().str() ) ) {
-        fclose(in);
-        regfree(&seekfor);
-
+           || ( dp.value != possMode.str() ) )
         return make_err(expect_mode_key,"mode").with_file(possModeFile, dp.line_num);
-      }
+
       get_sucess = getdata_pair(toParse, dp, buf);
       to_lower(dp.key);
       if (    !get_sucess
            || ( dp.key != "aspell" )
            || ( dp.value == NULL )
-           || ( *(dp.value) == '\0' ) ) { 
-        fclose(in);
-        regfree(&seekfor);
-
+           || ( *(dp.value) == '\0' ) )
         return make_err(mode_version_requirement).with_file(possModeFile, dp.line_num);
-      }
 
-      char * requirement = dp.value.str;
-      char * relop = requirement;
-      char swap = '\0';
-
-      if (    ( *requirement == '>' )
-           || ( *requirement == '<' )
-           || ( *requirement == '!' ) ) {
-        requirement++;
-      }
-      if ( *requirement == '=' ) {
-           requirement++;
-      }
-
-      String reqVers(requirement);
-
-      swap = *requirement;
-      *requirement = '\0';
-
-      String relOp(relop);
-
-      *requirement = swap;
-
-      char actVersion[] = PACKAGE_VERSION;
-      char * act = &actVersion[0];
-      char * seek = act;
-
-      while (    ( seek != NULL )
-              && ( *seek != '\0' ) 
-              && ( *seek < '0' )
-              && ( *seek > '9' ) 
-              && ( *seek != '.' )
-              && ( *seek != 'x' )
-              && ( *seek != 'X' ) ) {
-        seek++;
-      }
-      act = seek;
-      while (    ( seek != NULL )
-              && ( seek != '\0' ) 
-              && (    (    ( *seek >= '0' )
-                        && ( *seek <= '9' ) )
-                   || ( *seek == '.' )
-                   || ( *seek == 'x' )
-                   || ( *seek == 'X' ) ) ) {
-        seek++;
-      }
-      if ( seek != NULL ) {
-        *seek = '\0';
-      }
-
-      PosibErr<bool> peb = verifyVersion(relOp.str(),act,requirement,"add_filter");
-
-      if ( peb.has_err() ) {
-        peb.ignore_err();
-        fclose(in);
-        regfree(&seekfor);
-
-        return make_err(confusing_mode_version).with_file(possModeFile, dp.line_num);
-      }
-      if ( peb == false ) {
-        peb.ignore_err();
-        fclose(in);
-        regfree(&seekfor);
-
-        return make_err(bad_mode_version).with_file(possModeFile, dp.line_num);
-      }
+      PosibErr<void> peb = check_version(dp.value.str);
+      if (peb.has_err()) return peb.with_file(possModeFile, dp.line_num);
       
       FilterMode collect(possMode);
       
@@ -711,21 +603,15 @@ namespace acommon {
 
           char * regbegin = dp.value;
 
-
           while (    regbegin
                   && ( *regbegin != '/' ) ) {
             regbegin++;
           }
           if (    ( regbegin == NULL )
                || ( *regbegin == '\0' ) 
-               || ( *(++regbegin) == '\0' ) ) {
-            fclose(in);
-            regfree(&seekfor);
-
+               || ( *(++regbegin) == '\0' ) )
             return make_err(missing_magic_expression).with_file(possModeFile, dp.line_num);
-          }
           
-
           char * regend = regbegin;
           bool prevslash = false;
 
@@ -741,12 +627,8 @@ namespace acommon {
             }
             regend ++ ;
           }
-          if ( regend == regbegin ) {
-            fclose(in);
-            regfree(&seekfor);
-
+          if ( regend == regbegin )
             return make_err(missing_magic_expression).with_file(possModeFile, dp.line_num);
-          }
 
           char swap = *regend;
 
@@ -766,12 +648,9 @@ namespace acommon {
                     && ( *regend != '\0' ) ) {
               regend++;
             }
-            if ( regend == regbegin ) {
-              fclose(in);
-              regfree(&seekfor);
-
+            if ( regend == regbegin ) 
+            {
               char charCount[64];
-
               sprintf(&charCount[0],"%i",regbegin - (char *)dp.value);
               return  make_err(empty_file_ext,charCount).with_file(possModeFile,dp.line_num);
             }
@@ -787,12 +666,9 @@ namespace acommon {
               remove = true;
               regbegin++;
             }
-            if ( regend == regbegin ) {
-              fclose(in);
-              regfree(&seekfor);
-
+            if ( regend == regbegin ) 
+            {
               char charCount[64];
-
               sprintf(&charCount[0],"%i",regbegin - (char *)dp.value);
               return  make_err(empty_file_ext,charCount).with_file(possModeFile,dp.line_num);
             }
@@ -817,60 +693,44 @@ namespace acommon {
 
             PosibErr<bool> pe;
 
-            if ( remove ) { 
+            if ( remove )
               pe = collect.remModeExtension(ext,magic);
-            }
-            else {
+            else
               pe = collect.addModeExtension(ext,magic);
-            }
-            if ( pe.has_err() ) {
-              fclose(in);
-              regfree(&seekfor);
 
+            if ( pe.has_err() )
               return pe.with_file(possModeFile, dp.line_num);
-            }
           }
-          if ( extCount > 0 ) {
-            continue;
-          }
-          fclose(in);
-          regfree(&seekfor);
-          
-          char charCount[64];
 
+          if (extCount > 0 ) continue;
+
+          char charCount[64];
           sprintf(&charCount[0],"%i",strlen((char *)dp.value));
           return  make_err(empty_file_ext,charCount).with_file(possModeFile,dp.line_num);
         }
-        fclose(in);
-        regfree(&seekfor);
 
         return make_err(expect_mode_key,"ext[tension]/magic/desc[ription]/rel[ation]")
           .with_file(possModeFile,dp.line_num);
       
       }//while getdata_pair
       
-      PosibErr<void> pe = collect.build(in,config,dp.line_num,possMode.str());
+      RET_ON_ERR(collect.build(toParse,dp.line_num,possMode.str()));
 
-      fclose(in);
-      if ( pe.has_err() ) {
-        regfree(&seekfor);
-        return PosibErrBase(pe);
-      }
-      filterModes.push_back(collect);
+      filter_modes->push_back(collect);
     }
-    regfree(&seekfor);
-    return no_err;
+    return filter_modes.release();
   }
 
-  void activate_filter_modes(Config *config) {
-
-    ModeNotifierImpl * activate = NULL;
-
-    config->add_notifier((activate = new ModeNotifierImpl(config)));
-//    activate->loadModes();//ensure existence of modes
+  void activate_filter_modes(Config *config) 
+  {
+    config->add_notifier(new ModeNotifierImpl(config));
   }
 
-  void print_mode_help(FILE * helpScreen) {
+  PosibErr<void> print_mode_help(const Config * config, FILE * helpScreen) {
+
+    RET_ON_ERR_SET(static_cast<ModeNotifierImpl *>(config->filter_mode_notifier)
+                   ->get_filter_modes(), FilterModeList *, fm);
+    
     fprintf(helpScreen,
       "\n\n[Filter Modes] reconfigured combinations filters optimized for files of\n"
           "               a specific type. A mode is selected by Aspell's `--mode\n"
@@ -880,17 +740,17 @@ namespace acommon {
           "         Note: If the file type can not be identified uniquely by the\n"
           "               file extension Aspell will in addition test the file\n"
           "               content to ensure proper mode selection.\n\n");
-    for ( Vector<FilterMode>::iterator it = filterModes.begin() ;
-          it != filterModes.end() ; it++ ) {
+    for (Vector<FilterMode>::iterator it = fm->begin(); it != fm->end(); it++)
+    {
       fprintf(helpScreen,"  %-10s ",(*it).modeName().str());
 
       String desc = (*it).getDescription();
-      int preLength = (*it).modeName().length() + 4;
+      int preLength = (*it).modeName().size() + 4;
 
       if ( preLength < 13 ) {
         preLength = 13;
       }
-      while ( desc.length() > 74 - preLength ) {
+      while ( (int)desc.size() > 74 - preLength ) {
 
         int locate = 74 - preLength;
 
@@ -906,10 +766,10 @@ namespace acommon {
         
         String prDesc(desc);
 
-        prDesc.erase(locate,prDesc.length() - locate);
+        prDesc.erase(locate,prDesc.size() - locate);
         fprintf(helpScreen,"%s\n             ",prDesc.str());
         desc.erase(0,locate);
-        if (    ( desc.length() > 0 )
+        if (    ( desc.size() > 0 )
              && (    ( desc[0] == ' ' )
                   || ( desc[0] == '\t' )
                   || ( desc[0] == '\n' ) ) ) {
@@ -920,16 +780,7 @@ namespace acommon {
       fprintf(helpScreen,desc.str());
       fprintf(helpScreen,"\n");
     }
+    return no_err;
   }
-
-  PosibErr<void> intialize_filter_modes(Config * config){
-    filterModesRef++;
-
-    ModeNotifierImpl * intializer = new ModeNotifierImpl(config);
-
-    PosibErr<void> init_err = intializer->initModes();
-    filterModesRef--;
-    return init_err;
-  }
-};
+}
 
